# Copyright (C) 2011 by Barry Warsaw
#
# This file is part of Taster.
#
# Taster is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Taster is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# Taster.  If not, see <http://www.gnu.org/licenses/>.

"""Download and hash check packages."""

__all__ = [
    ]


import os
import json
import hashlib
import argparse

from urllib.request import urlopen


PACKAGE_URL = 'http://pypi.python.org/pypi/{0}/json'


class ChecksumMismatchError(Exception):
    def __init__(self, expected, got):
        self.expected = expected
        self.got = got

    def __str__(self):
        return 'got: {0.got} != expected: {0.expected}'.format(self)


class NotAnSdistError(Exception):
    pass


class Downloader:
    def __init__(self, working_dir=None):
        self.working_dir = (working_dir if working_dir is not None else '.')

    def get_package(self, name):
        url = PACKAGE_URL.format(name)
        with urlopen(url) as fp:
            # In Python 3, json.load only accepts strings, but you can't get
            # strings out of the opener's file object.  I think it's probably
            # a safe assumption that PyPI returns utf-8 encoded json bytes.
            data = fp.read().decode('utf-8')
            json_data = json.loads(data)
        assert len(json_data['urls']) == 1, 'unexpected entry count'
        url_entries = json_data['urls'][0]
        source_url = url_entries['url']
        md5_digest = url_entries['md5_digest']
        filename = url_entries['filename']
        package_type = url_entries['packagetype']
        if package_type != 'sdist':
            raise NotAnSdistError
        md5 = hashlib.md5()
        outpath = os.path.join(self.working_dir, filename)
        with urlopen(source_url) as infp:
            with open(outpath, 'wb') as outfp:
                data = infp.read()
                md5.update(data)
                outfp.write(data)
        return filename, md5_digest, md5.hexdigest()

    def download(self, name):
        filename, md5_expected, md5_got = self.get_package(name)
        if md5_expected != md5_got:
            raise ChecksumMismatchError(md5_expected, md5_got)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '-d', '--directory',
        help='Working directory')
    parser.add_argument(
        'package_name',
        help='The name of the package to download')
    args = parser.parse_args()
    downloader = Downloader(args.directory)
    downloader.download(args.package_name)
    

if __name__ == '__main__':
    main()
