# Copyright (C) 2011 by Barry Warsaw
#
# This file is part of Taster.
#
# Taster is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Taster is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# Taster.  If not, see <http://www.gnu.org/licenses/>.

"""Unpacking."""

__all__ = [
    ]


import tarfile
import zipfile
import argparse
import mimetypes


class UnknownTypeError(Exception):
    def __init__(self, path, content_type):
        self.path = path
        self.content_type = content_type

    def __str__(self):
        return 'Unknown content-type {0.content_type}: {0.path}'.format(self)


class Unpacker:
    """Unpack a source dist into a working directory."""

    def __init__(self, working_dir=None):
        self.working_dir = (working_dir if working_dir is not None else '.')

    def unpack(self, sdistpath):
        # What kind of file are we talking about?  zip, tar.gz, tgz, tar.bz2
        content_type, encoding = mimetypes.guess_type(sdistpath)
        if content_type is None:
            main_type = sub_type = None
        else:
            main_type, slash, sub_type = content_type.partition('/')
        if main_type not in ('application', None):
            raise UnknownTypeError(sdistpath, content_type)
        if sub_type == 'x-tar':
            # tarfile
            archive = tarfile.open(sdistpath)
        elif sub_type == 'zip':
            # zip
            archive = zipfile.ZipFile(sdistpath)
        else:
            raise UnknownTypeError(sdistpath, content_type)
        archive.extractall(self.working_dir)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '-d', '--directory',
        help='Working directory')
    parser.add_argument(
        'sdistpath',
        help='The path to the sdist to unpack.')
    args = parser.parse_args()
    unpacker = Unpacker(args.directory)
    unpacker.unpack(args.sdistpath)


if __name__ == '__main__':
    main()
